# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

# flake8: noqa
"""
AISec API service

OpenAPI Specification for the AI Runtime Security API service

The version of the OpenAPI document: 0.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

# import models into model package
from aisecurity.generated_openapi_client.models.agent_entry_object import AgentEntryObject
from aisecurity.generated_openapi_client.models.agent_meta import AgentMeta
from aisecurity.generated_openapi_client.models.agent_report_object import AgentReportObject
from aisecurity.generated_openapi_client.models.ai_profile import AiProfile
from aisecurity.generated_openapi_client.models.async_scan_object import AsyncScanObject
from aisecurity.generated_openapi_client.models.async_scan_response import AsyncScanResponse
from aisecurity.generated_openapi_client.models.cg_report_object import CgReportObject
from aisecurity.generated_openapi_client.models.cmd_entry_object import CmdEntryObject
from aisecurity.generated_openapi_client.models.ds_detail_result_object import DSDetailResultObject
from aisecurity.generated_openapi_client.models.ds_result_metadata import DSResultMetadata
from aisecurity.generated_openapi_client.models.dbs_entry_object import DbsEntryObject
from aisecurity.generated_openapi_client.models.detection_service_result_object import DetectionServiceResultObject
from aisecurity.generated_openapi_client.models.dlp_pattern_detections_object import DlpPatternDetectionsObject
from aisecurity.generated_openapi_client.models.dlp_report_object import DlpReportObject
from aisecurity.generated_openapi_client.models.error import Error
from aisecurity.generated_openapi_client.models.io_detected import IODetected
from aisecurity.generated_openapi_client.models.malware_report_object import MalwareReportObject
from aisecurity.generated_openapi_client.models.masked_data import MaskedData
from aisecurity.generated_openapi_client.models.mc_entry_object import McEntryObject
from aisecurity.generated_openapi_client.models.mc_report_object import McReportObject
from aisecurity.generated_openapi_client.models.metadata import Metadata
from aisecurity.generated_openapi_client.models.pattern_detections import PatternDetections
from aisecurity.generated_openapi_client.models.prompt_detected import PromptDetected
from aisecurity.generated_openapi_client.models.prompt_detection_details import PromptDetectionDetails
from aisecurity.generated_openapi_client.models.response_detected import ResponseDetected
from aisecurity.generated_openapi_client.models.response_detection_details import ResponseDetectionDetails
from aisecurity.generated_openapi_client.models.scan_id_result import ScanIdResult
from aisecurity.generated_openapi_client.models.scan_request import ScanRequest
from aisecurity.generated_openapi_client.models.scan_request_contents_inner import ScanRequestContentsInner
from aisecurity.generated_openapi_client.models.scan_response import ScanResponse
from aisecurity.generated_openapi_client.models.scan_summary import ScanSummary
from aisecurity.generated_openapi_client.models.scan_sync_request_default_response import ScanSyncRequestDefaultResponse
from aisecurity.generated_openapi_client.models.tc_report_object import TcReportObject
from aisecurity.generated_openapi_client.models.tg_report_object import TgReportObject
from aisecurity.generated_openapi_client.models.threat_scan_report_object import ThreatScanReportObject
from aisecurity.generated_openapi_client.models.tool_detected import ToolDetected
from aisecurity.generated_openapi_client.models.tool_detection_details import ToolDetectionDetails
from aisecurity.generated_openapi_client.models.tool_detection_entry import ToolDetectionEntry
from aisecurity.generated_openapi_client.models.tool_detection_flags import ToolDetectionFlags
from aisecurity.generated_openapi_client.models.tool_event import ToolEvent
from aisecurity.generated_openapi_client.models.tool_event_metadata import ToolEventMetadata
from aisecurity.generated_openapi_client.models.topic_guard_rails import TopicGuardRails
from aisecurity.generated_openapi_client.models.urlf_entry_object import UrlfEntryObject



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

"""
AISec API service

OpenAPI Specification for the AI Runtime Security API service

The version of the OpenAPI document: 0.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator, model_validator
from typing import Any, ClassVar, Dict, List
from aisecurity.generated_openapi_client.models.ai_profile import AiProfile
from aisecurity.generated_openapi_client.models.metadata import Metadata
from aisecurity.generated_openapi_client.models.scan_request_contents_inner import ScanRequestContentsInner
from aisecurity.constants.base import MAX_TRANSACTION_ID_STR_LENGTH, MAX_SESSION_ID_STR_LENGTH
from aisecurity.exceptions import AISecSDKException, ErrorType
from typing import Optional, Set
from typing_extensions import Self


class ScanRequest(BaseModel):
    """
    ScanRequest
    """  # noqa: E501

    tr_id: Optional[StrictStr] = Field(
        default=None, description="Unique identifier for the transaction correlating prompt and response"
    )
    session_id: Optional[StrictStr] = Field(default=None, description="Unique identifier for tracking Sessions")
    ai_profile: AiProfile
    metadata: Optional[Metadata] = None
    contents: List[ScanRequestContentsInner] = Field(
        description="List of prompt or response or prompt/response pairs. The last element is the one that needs to be scanned, and the previous elements are the context for the scan."
    )
    __properties: ClassVar[List[str]] = ["tr_id", "session_id", "ai_profile", "metadata", "contents"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    @field_validator("session_id")
    @classmethod
    def validate_id_length(cls, v: Optional[str], info) -> Optional[str]:
        """Only validate session_id length"""
        if v is not None:
            cleaned_value = str(v).strip()
            if len(cleaned_value) > MAX_SESSION_ID_STR_LENGTH:
                raise AISecSDKException(
                    f"sessionID exceeds maximum length of {MAX_SESSION_ID_STR_LENGTH} characters",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
            return cleaned_value
        return v

    @model_validator(mode="after")
    def validate_id_priority(self) -> "ScanRequest":
        """Handle priority logic: session_id > tr_id, but only validate tr_id if session_id is empty"""
        session_id = self.session_id
        tr_id = self.tr_id

        # Check if session_id has actual content
        has_valid_session_id = session_id and str(session_id).strip()

        # If session_id is empty/None but tr_id is provided, validate tr_id length
        if not has_valid_session_id and tr_id:
            cleaned_tr_id = str(tr_id).strip()
            if len(cleaned_tr_id) > MAX_TRANSACTION_ID_STR_LENGTH:
                raise AISecSDKException(
                    f"transactionID exceeds maximum length of {MAX_TRANSACTION_ID_STR_LENGTH} characters",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )

        return self

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ScanRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of ai_profile
        if self.ai_profile:
            _dict["ai_profile"] = self.ai_profile.to_dict()
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict["metadata"] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in contents (list)
        _items = []
        if self.contents:
            for _item_contents in self.contents:
                if _item_contents:
                    _items.append(_item_contents.to_dict())
            _dict["contents"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ScanRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "tr_id": obj.get("tr_id"),
            "session_id": obj.get("session_id"),
            "ai_profile": AiProfile.from_dict(obj["ai_profile"]) if obj.get("ai_profile") is not None else None,
            "metadata": Metadata.from_dict(obj["metadata"]) if obj.get("metadata") is not None else None,
            "contents": [ScanRequestContentsInner.from_dict(_item) for _item in obj["contents"]]
            if obj.get("contents") is not None
            else None,
        })
        return _obj



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

# flake8: noqa

# import apis into api package
from aisecurity.generated_openapi_client.urllib3.api.scan_reports_api import (
    ScanReportsApi,
)



# Copyright (c) 2025, Palo Alto Networks

# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.

# You may obtain a copy of the License at:

# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md

# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

import json
from aisecurity.constants.base import (
    MAX_CONTENT_CONTEXT_LENGTH,
    MAX_CONTENT_PROMPT_LENGTH,
    MAX_CONTENT_RESPONSE_LENGTH,
)
from aisecurity.exceptions import AISecSDKException, ErrorType
from aisecurity.logger import BaseLogger
from aisecurity.generated_openapi_client.models.tool_event import ToolEvent


class Content(BaseLogger):
    def __init__(self, prompt=None, response=None, context=None, code_prompt=None, code_response=None, tool_event=None):
        super().__init__()
        self._prompt = None
        self._response = None
        self._context = None
        self._code_prompt = None
        self._code_response = None
        self._tool_event = None
        self.prompt = prompt  # This will trigger the setter with the length check
        self.response = response  # This will trigger the setter with the length check
        self.context = context
        self.code_prompt = code_prompt
        self.code_response = code_response
        self.tool_event = tool_event

        if (
            not self.prompt
            and not self.response
            and not self.code_prompt
            and not self.code_response
            and not self.tool_event
        ):
            error_msg = "content validation failed: at least one of Prompt, Response, CodePrompt, CodeResponse, or ToolEvent must be provided"
            raise AISecSDKException(
                error_msg,
                ErrorType.USER_REQUEST_PAYLOAD_ERROR,
            )

    @property
    def prompt(self):
        return self._prompt

    @prompt.setter
    def prompt(self, value):
        if value is not None:
            if not isinstance(value, str):
                raise AISecSDKException(
                    "Prompt must be of type str",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
            if len(value) > MAX_CONTENT_PROMPT_LENGTH:
                raise AISecSDKException(
                    f"Prompt length exceeds maximum allowed length of {MAX_CONTENT_PROMPT_LENGTH} characters",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
        self._prompt = value

    @property
    def response(self):
        return self._response

    @response.setter
    def response(self, value):
        if value is not None:
            if not isinstance(value, str):
                raise AISecSDKException(
                    "Response must be of type str",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
            if len(value) > MAX_CONTENT_RESPONSE_LENGTH:
                raise AISecSDKException(
                    f"Response length exceeds maximum allowed length of {MAX_CONTENT_RESPONSE_LENGTH} characters",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
        self._response = value

    @property
    def context(self):
        return self._context

    @context.setter
    def context(self, value):
        if value is not None:
            if not isinstance(value, str):
                raise AISecSDKException(
                    "Context must be of type str",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
            if len(value) > MAX_CONTENT_CONTEXT_LENGTH:
                raise AISecSDKException(
                    f"context length exceeds maximum allowed length of {MAX_CONTENT_CONTEXT_LENGTH} characters",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
        self._context = value

    @property
    def code_prompt(self):
        return self._code_prompt

    @code_prompt.setter
    def code_prompt(self, value):
        if value is not None:
            if not isinstance(value, str):
                raise AISecSDKException(
                    "Code prompt must be of type str",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
            if len(value) > MAX_CONTENT_PROMPT_LENGTH:
                raise AISecSDKException(
                    f"Code prompt length exceeds maximum allowed length of {MAX_CONTENT_PROMPT_LENGTH} characters",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
        self._code_prompt = value

    @property
    def code_response(self):
        return self._code_response

    @code_response.setter
    def code_response(self, value):
        if value is not None:
            if not isinstance(value, str):
                raise AISecSDKException(
                    "Code response must be of type str",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
            if len(value) > MAX_CONTENT_RESPONSE_LENGTH:
                raise AISecSDKException(
                    f"Code response length exceeds maximum allowed length of {MAX_CONTENT_RESPONSE_LENGTH} characters",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
        self._code_response = value

    @property
    def tool_event(self):
        return self._tool_event

    @tool_event.setter
    def tool_event(self, value):
        if value is not None and not isinstance(value, ToolEvent):
            raise AISecSDKException(
                "tool_event must be an instance of ToolEvent",
                ErrorType.USER_REQUEST_PAYLOAD_ERROR,
            )
        self._tool_event = value

    def to_json(self):
        tool_event_dict = None
        if self._tool_event is not None:
            tool_event_dict = self._tool_event.to_dict()

        return json.dumps({
            "prompt": self._prompt,
            "response": self._response,
            "context": self._context,
            "code_prompt": self._code_prompt,
            "code_response": self._code_response,
            "tool_event": tool_event_dict,
        })

    @classmethod
    def from_json(cls, json_str):
        if json_str is None:
            return None

        data = json.loads(json_str)
        tool_event = None
        if data.get("tool_event"):
            tool_event = ToolEvent.from_dict(data.get("tool_event"))

        return cls(
            prompt=data.get("prompt"),
            response=data.get("response"),
            context=data.get("context"),
            code_prompt=data.get("code_prompt"),
            code_response=data.get("code_response"),
            tool_event=tool_event,
        )

    @classmethod
    def from_json_file(cls, file_path):
        if file_path is None:
            raise AISecSDKException(
                "File path cannot be None",
                ErrorType.USER_REQUEST_PAYLOAD_ERROR,
            )

        try:
            with open(file_path) as json_file:
                data = json.load(json_file)
        except FileNotFoundError:
            raise AISecSDKException(
                f"File not found: {file_path}",
                ErrorType.USER_REQUEST_PAYLOAD_ERROR,
            )
        except (IOError, OSError) as e:
            raise AISecSDKException(
                f"Error reading file: {str(e)}",
                ErrorType.USER_REQUEST_PAYLOAD_ERROR,
            )

        tool_event = None
        if data.get("tool_event"):
            tool_event = ToolEvent.from_dict(data.get("tool_event"))

        return cls(
            prompt=data.get("prompt"),
            response=data.get("response"),
            context=data.get("context"),
            code_prompt=data.get("code_prompt"),
            code_response=data.get("code_response"),
            tool_event=tool_event,
        )

    def __len__(self):
        tool_event_length = 0
        if self._tool_event is not None:
            # Calculate length based on ToolEvent's input and output fields
            tool_event_length = len(self._tool_event.input or "") + len(self._tool_event.output or "")

        return (
            len(self._prompt or "")
            + len(self._response or "")
            + len(self._context or "")
            + len(self._code_prompt or "")
            + len(self._code_response or "")
            + tool_event_length
        )

    def __str__(self):
        return f"Content(prompt={self._prompt}, response={self._response}, context={self._context}, code_prompt={self._code_prompt}, code_response={self._code_response}, tool_event={self._tool_event})"



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

"""
AISec API service

OpenAPI Specification for the AI Runtime Security API service

The version of the OpenAPI document: 0.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict
from typing import Any, ClassVar, Dict, List, Optional
from aisecurity.generated_openapi_client.models.agent_report_object import AgentReportObject
from aisecurity.generated_openapi_client.models.cg_report_object import CgReportObject
from aisecurity.generated_openapi_client.models.dbs_entry_object import DbsEntryObject
from aisecurity.generated_openapi_client.models.dlp_report_object import DlpReportObject
from aisecurity.generated_openapi_client.models.mc_report_object import McReportObject
from aisecurity.generated_openapi_client.models.tc_report_object import TcReportObject
from aisecurity.generated_openapi_client.models.tg_report_object import TgReportObject
from aisecurity.generated_openapi_client.models.urlf_entry_object import UrlfEntryObject
from typing import Optional, Set
from typing_extensions import Self


class DSDetailResultObject(BaseModel):
    """
    DSDetailResultObject
    """  # noqa: E501

    urlf_report: Optional[List[UrlfEntryObject]] = None
    dlp_report: Optional[DlpReportObject] = None
    dbs_report: Optional[List[DbsEntryObject]] = None
    tc_report: Optional[TcReportObject] = None
    mc_report: Optional[McReportObject] = None
    agent_report: Optional[AgentReportObject] = None
    topic_guardrails_report: Optional[TgReportObject] = None
    cg_report: Optional[CgReportObject] = None
    __properties: ClassVar[List[str]] = [
        "urlf_report",
        "dlp_report",
        "dbs_report",
        "tc_report",
        "mc_report",
        "agent_report",
        "topic_guardrails_report",
        "cg_report",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of DSDetailResultObject from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in urlf_report (list)
        _items = []
        if self.urlf_report:
            for _item_urlf_report in self.urlf_report:
                if _item_urlf_report:
                    _items.append(_item_urlf_report.to_dict())
            _dict["urlf_report"] = _items
        # override the default output from pydantic by calling `to_dict()` of dlp_report
        if self.dlp_report:
            _dict["dlp_report"] = self.dlp_report.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in dbs_report (list)
        _items = []
        if self.dbs_report:
            for _item_dbs_report in self.dbs_report:
                if _item_dbs_report:
                    _items.append(_item_dbs_report.to_dict())
            _dict["dbs_report"] = _items
        # override the default output from pydantic by calling `to_dict()` of tc_report
        if self.tc_report:
            _dict["tc_report"] = self.tc_report.to_dict()
        # override the default output from pydantic by calling `to_dict()` of mc_report
        if self.mc_report:
            _dict["mc_report"] = self.mc_report.to_dict()
        # override the default output from pydantic by calling `to_dict()` of agent_report
        if self.agent_report:
            _dict["agent_report"] = self.agent_report.to_dict()
        # override the default output from pydantic by calling `to_dict()` of topic_guardrails_report
        if self.topic_guardrails_report:
            _dict["topic_guardrails_report"] = self.topic_guardrails_report.to_dict()
        # override the default output from pydantic by calling `to_dict()` of cg_report
        if self.cg_report:
            _dict["cg_report"] = self.cg_report.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DSDetailResultObject from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "urlf_report": [UrlfEntryObject.from_dict(_item) for _item in obj["urlf_report"]]
            if obj.get("urlf_report") is not None
            else None,
            "dlp_report": DlpReportObject.from_dict(obj["dlp_report"]) if obj.get("dlp_report") is not None else None,
            "dbs_report": [DbsEntryObject.from_dict(_item) for _item in obj["dbs_report"]]
            if obj.get("dbs_report") is not None
            else None,
            "tc_report": TcReportObject.from_dict(obj["tc_report"]) if obj.get("tc_report") is not None else None,
            "mc_report": McReportObject.from_dict(obj["mc_report"]) if obj.get("mc_report") is not None else None,
            "agent_report": AgentReportObject.from_dict(obj["agent_report"])
            if obj.get("agent_report") is not None
            else None,
            "topic_guardrails_report": TgReportObject.from_dict(obj["topic_guardrails_report"])
            if obj.get("topic_guardrails_report") is not None
            else None,
            "cg_report": CgReportObject.from_dict(obj["cg_report"]) if obj.get("cg_report") is not None else None,
        })
        return _obj



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

from singleton_decorator import singleton

from aisecurity.constants.base import MAX_NUMBER_OF_REPORT_IDS, MAX_REPORT_ID_STR_LENGTH
from aisecurity.exceptions import AISecSDKException, ErrorType
from aisecurity.generated_openapi_client import ScanReportsApi, ThreatScanReportObject
from aisecurity.generated_openapi_client.urllib3.exceptions import ApiException
from aisecurity.scan.inline.base import ApiBase


@singleton
class QueryByReportIds(ApiBase):
    """
    A singleton class for querying threat scan reports by report IDs.
    """

    def __init__(self):
        """
        Initialize the QueryByReportId instance with the ScanReportsApi.

        This method sets up the API client for making requests to the ScanReportsApi.
        """
        super().__init__()
        self.__api = ScanReportsApi(api_client=self.api_client)

    def get_threat_objects(self, report_ids: list[str]) -> list[ThreatScanReportObject]:
        """
        Retrieve threat scan reports for the given report IDs.

        This method queries the ScanReportsApi to fetch threat scan reports
        corresponding to the provided report IDs.

        Args:
            report_ids (list[str]): A list of report IDs to query.

        Returns:
            list[ThreatScanReportObject]: A list of threat scan report objects
                corresponding to the provided report IDs.

        Raises:
            ValueError: If any report ID exceeds the maximum allowed length.
        """
        if not report_ids or len(report_ids) == 0:
            error_msg = "At least one report ID must be provided"
            self.logger.error(f"event={self.get_threat_objects.__name__} error={error_msg}")
            raise AISecSDKException(
                "At least one report ID must be provided",
                ErrorType.USER_REQUEST_PAYLOAD_ERROR,
            )

        if len(report_ids) > MAX_NUMBER_OF_REPORT_IDS:
            error_msg = f"The number of report_ids should not exceed {MAX_NUMBER_OF_REPORT_IDS}."
            self.logger.error(f"event={self.get_threat_objects.__name__} error={error_msg}")
            raise AISecSDKException(
                error_msg,
                ErrorType.USER_REQUEST_PAYLOAD_ERROR,
            )

        # Validate report ID lengths
        for report_id in report_ids:
            if report_id is None or len(report_id) == 0:
                raise AISecSDKException(
                    "Report ID Can't be None or Empty",
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )
            elif len(report_id) > MAX_REPORT_ID_STR_LENGTH:
                error_msg = (
                    f"Report ID '{report_id}' exceeds the maximum allowed length of "
                    f"{MAX_REPORT_ID_STR_LENGTH} characters."
                )
                self.logger.error(f"event={self.get_threat_objects.__name__} error={error_msg}")
                raise AISecSDKException(
                    error_msg,
                    ErrorType.USER_REQUEST_PAYLOAD_ERROR,
                )

        try:
            result = self.__api.get_threat_scan_reports(report_ids=report_ids)
            return result
        except ApiException as e:
            """
                Handle API exceptions that may occur during the API call. Ex HTTP Server side Errors,API Key errors,API value errors,Authorization errors etc.
            """
            self.logger.error(f"event={self.get_threat_objects.__name__} error={e}")
            raise AISecSDKException(str(e), ErrorType.SERVER_SIDE_ERROR)
        except Exception as e:
            self.logger.error(f"event={self.get_threat_objects.__name__} error={e!r}")
            raise AISecSDKException(str(e), ErrorType.AISEC_SDK_ERROR)



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.
from typing import Optional

import aiohttp
from singleton_decorator import singleton

from aisecurity.exceptions import AISecSDKException, ErrorType
from aisecurity.generated_openapi_client import (
    AiProfile,
    ScanRequest,
    ScanRequestContentsInner,
    ScanResponse,
)
from aisecurity.generated_openapi_client.asyncio.exceptions import ApiException
from aisecurity.generated_openapi_client.models.metadata import Metadata
from aisecurity.scan.asyncio.base import ScanApiBase
from aisecurity.scan.models.content import Content


@singleton
class ScanExecutor(ScanApiBase):
    def __init__(self):
        """
        Initialize the ScanExecutor with the ScansApi client.
        """
        super().__init__()

    async def sync_request(
        self,
        content: Content,
        ai_profile: AiProfile,
        tr_id: Optional[str],
        session_id: Optional[str],
        metadata: Optional[Metadata],
    ) -> ScanResponse:
        """
        Create and execute a synchronous scan request.

        Args:
            content (Content): The content to be scanned.
            ai_profile (AiProfile): The AI profile to be used for scanning.
            tr_id (str): Optionally Provide any unique identifier string for correlating the prompt and response transactions. This is an optional field. The tr_id value received for scan request is returned in the scan response along with the scan ID
            session_id (str): Optionally send session_id to track session views
            metadata (Metadata): Optionally send the app_name, app_user, and ai_model in the metadata

        Returns:
            ScanResponse: The response from the synchronous scan request.
        """
        try:
            return await self.scan_api.scan_sync_request(
                scan_request=ScanRequest(
                    tr_id=tr_id,
                    session_id=session_id,
                    contents=[
                        ScanRequestContentsInner(
                            prompt=content.prompt,
                            response=content.response,
                            context=content.context,
                            code_prompt=content.code_prompt,
                            code_response=content.code_response,
                            tool_event=content.tool_event,
                        )
                    ],
                    ai_profile=ai_profile,
                    metadata=metadata,
                ),
            )
        except ApiException as e:
            """
                Handle API exceptions that may occur during the API call
            """
            self.logger.error(f"event={self.sync_request.__name__} error={e}")
            raise AISecSDKException(str(e), ErrorType.SERVER_SIDE_ERROR)
        except aiohttp.ClientError as e:
            """
                A catch-all for other client-side exceptions that may occur during the API call
            """
            self.logger.error(f"event={self.sync_request.__name__} error={e}")
            raise AISecSDKException(str(e), ErrorType.CLIENT_SIDE_ERROR)
        except Exception as e:
            self.logger.error(f"event={self.sync_request.__name__} error={e}")
            raise AISecSDKException(str(e))



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

import os
import warnings
from typing import Optional

from aisecurity.constants.base import (
    AI_SEC_API_ENDPOINT,
    AI_SEC_API_KEY,
    AI_SEC_API_TOKEN,
    DEFAULT_ENDPOINT,
    MAX_API_KEY_LENGTH,
    MAX_NUMBER_OF_RETRIES,
    MAX_TOKEN_LENGTH,
)
from aisecurity.exceptions import AISecSDKException, ErrorType
from aisecurity.logger import BaseLogger


class _Configuration(BaseLogger):
    def __init__(self):
        super().__init__()
        self._api_endpoint = DEFAULT_ENDPOINT
        self._api_key = None
        self._api_token = None
        self._num_retries = MAX_NUMBER_OF_RETRIES

    def init(
        self,
        *,
        api_key: Optional[str] = None,
        api_token: Optional[str] = None,
        api_endpoint: Optional[str] = None,
        num_retries: Optional[int] = None,
    ):
        if api_endpoint:
            self.api_endpoint = api_endpoint
        elif api_endpoint := os.getenv(AI_SEC_API_ENDPOINT):
            self.api_endpoint = api_endpoint

        if api_key:
            self.api_key = api_key
        elif api_key := os.getenv(AI_SEC_API_KEY):
            self.api_key = api_key

        if api_token:
            self.api_token = api_token
        elif api_token := os.getenv(AI_SEC_API_TOKEN):
            self.api_token = api_token

        if not (self.api_key or self.api_token):
            self._log_and_raise(
                "Either api_key or api_token must be provided ",
                ErrorType.MISSING_VARIABLE,
            )

        if num_retries is not None:
            self.num_retries = num_retries

    def _log_and_raise(self, message, error_type):
        self.logger.error(f"event={self.init.__name__} {message}")
        raise AISecSDKException(message, error_type)

    @property
    def api_endpoint(self):
        return self._api_endpoint

    @api_endpoint.setter
    def api_endpoint(self, value):
        if value is None:
            value = DEFAULT_ENDPOINT
        self._api_endpoint = value
        self.logger.info(f"event={self.init.__name__} api_endpoint={self._api_endpoint} action=set")

    @property
    def api_key(self):
        return self._api_key

    @api_key.setter
    def api_key(self, value):
        if value is None or len(value) == 0:
            self._log_and_raise(
                "api_key can't be None",
                ErrorType.MISSING_VARIABLE,
            )
        if len(value) > MAX_API_KEY_LENGTH:
            self._log_and_raise(
                f"api_key can't exceed {MAX_API_KEY_LENGTH} bytes",
                ErrorType.AISEC_SDK_ERROR,
            )

        if self.api_token:
            warnings.warn("Both API key and OAuth token are configured. Consider using only one authentication method.")

        self._api_key = value
        self.logger.info(f"event={self.init.__name__} api_key value configured action=set")
        self.logger.debug(f"event={self.init.__name__} api_key_last8={self._api_key[:-8]}********* action=set")

    @property
    def num_retries(self):
        return self._num_retries

    @num_retries.setter
    def num_retries(self, value):
        if not isinstance(value, int):
            raise AISecSDKException(
                f"Invalid num_retries value: {value}. num_retries must be an integer.",
                ErrorType.AISEC_SDK_ERROR,
            )
        if value < 0:
            raise AISecSDKException(
                f"Invalid num_retries value: {value}. num_retries must be a non-negative integer.",
                ErrorType.AISEC_SDK_ERROR,
            )
        self._num_retries = value
        self.logger.info(f"event={self.init.__name__} var={self._num_retries} action=set")

    @property
    def api_token(self):
        return self._api_token

    @api_token.setter
    def api_token(self, value):
        if value is None or len(value) == 0:
            self._log_and_raise(
                "api_token can't be None",
                ErrorType.MISSING_VARIABLE,
            )
        if len(value) > MAX_TOKEN_LENGTH:
            self._log_and_raise(
                f"api_token can't exceed {MAX_TOKEN_LENGTH} bytes",
                ErrorType.AISEC_SDK_ERROR,
            )
        if self.api_key:
            warnings.warn("Both API key and OAuth token are configured. Consider using only one authentication method.")

        self._api_token = value
        self.logger.info(f"event={self.init.__name__} api_token value configured action=set")
        self.logger.debug(f"event={self.init.__name__} api_token_last8={self._api_token[:-8]}********* action=set")

    def reset(self):
        self._api_endpoint = DEFAULT_ENDPOINT
        self._api_key = None
        self._api_token = None
        self._num_retries = MAX_NUMBER_OF_RETRIES
        self.logger.info(f"event={self.reset.__name__} action=configuration_reset")


# TODO: Move away global/singleton configuration
global_configuration = _Configuration()



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

from enum import Enum
from typing import Optional


class ErrorType(Enum):
    """Enum defining error type codes for AISecSDK exceptions.

    These error types are used to categorize different errors that can occur
    within the SDK and are included in exception messages.
    """

    SERVER_SIDE_ERROR = "AISEC_SERVER_SIDE_ERROR"
    CLIENT_SIDE_ERROR = "AISEC_CLIENT_SIDE_ERROR"
    USER_REQUEST_PAYLOAD_ERROR = "AISEC_USER_REQUEST_PAYLOAD_ERROR"
    MISSING_VARIABLE = "AISEC_MISSING_VARIABLE"
    AISEC_SDK_ERROR = "AISEC_SDK_ERROR"


class AISecSDKException(Exception):
    """Base exception class for SDK-related exceptions."""

    def __init__(self, message: str = "", error_type: Optional[ErrorType] = None) -> None:
        self.message = message
        self.error_type = error_type

    def __str__(self) -> str:
        if self.error_type:
            return f"{self.error_type.value}:{self.message}"
        return f"{self.message}"



# Copyright (c) 2025, Palo Alto Networks
#
# Licensed under the Polyform Internal Use License 1.0.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
# https://polyformproject.org/licenses/internal-use/1.0.0
# (or)
# https://github.com/polyformproject/polyform-licenses/blob/76a278c4/PolyForm-Internal-Use-1.0.0.md
#
# As far as the law allows, the software comes as is, without any warranty
# or condition, and the licensor will not be liable to you for any damages
# arising out of these terms or the use or nature of the software, under
# any kind of legal claim.

"""API response object."""

from __future__ import annotations

from typing import Generic, Mapping, Optional, TypeVar

from pydantic import BaseModel, Field, StrictBytes, StrictInt

T = TypeVar("T")


class ApiResponse(BaseModel, Generic[T]):
    """
    API response object
    """

    status_code: StrictInt = Field(description="HTTP status code")
    headers: Optional[Mapping[str, str]] = Field(None, description="HTTP headers")
    data: T = Field(description="Deserialized data given the data type")
    raw_data: StrictBytes = Field(description="Raw data (HTTP response body)")

    model_config = {"arbitrary_types_allowed": True}
